#include "DiagonalPreconditioner.h"
#include "FiniteElemSpace.h"
#include "DOFAdmin.h"
#include "DOFVector.h"
#include "DOFMatrix.h"
#include "DOFIterator.h"

namespace AMDiS {

  void DiagonalPreconditioner::precon(DOFVector<double>* x)
  {
    FUNCNAME("DiagonalPreconditioner::precon()");

    TEST_EXIT_DBG(matrix[row])("no matrix\n");
    TEST_EXIT_DBG(x)("no solution vector\n");
 
    DOFMatrix::Iterator rowIterator((*matrix[row]), USED_DOFS);
    DOFVector<double>::Iterator vecIterator(x, USED_DOFS);

    if (bound) {
      ERROR_EXIT("not up to date\n");
      DOFVector<BoundaryType>::Iterator bIterator(const_cast<DOFVector<BoundaryType>*>(bound), USED_DOFS);
      for (vecIterator.reset(), rowIterator.reset(), bIterator.reset();
	   !vecIterator.end(); ++vecIterator, ++bIterator, ++rowIterator) {
	// only non-dirichlet nodes will be preconditioned
	if ((*bIterator) <= 0 && rowIterator->size() != 0) {
	  (*vecIterator) /= (*rowIterator)[0].entry;
	}
      }
    } else {
      for (vecIterator.reset(), rowIterator.reset(); 
	   !vecIterator.end(); 
	   ++vecIterator, ++rowIterator) {
	if (rowIterator->size() != 0) {
	  (*vecIterator) /= (*rowIterator)[0].entry;
	}
      }    
    }
  }
    
  void DiagonalPreconditionerStd::precon(std::vector<double>* x)
  {
    FUNCNAME("DiagonalPreconditionerStd::precon()");
	
    TEST_EXIT_DBG(x)("no solution vector\n");
    TEST_EXIT_DBG(x->size() == matrix->size())("solution vector and matrix have different size\n");
    
    std::vector<double>::iterator vecIt;
    std::vector< std::vector<MatEntry> >::iterator matrixIt;
      
    for (vecIt = x->begin(), matrixIt = matrix->begin(); 
	 vecIt < x->end(); 
	 ++vecIt, ++matrixIt) {
      if (matrixIt->size() != 0) {
	(*vecIt) /= (*matrixIt)[0].entry;
      }
    }
  }  
}
